package br.UFSC.GRIMA.entidades;

import br.UFSC.GRIMA.exceptions.MissingDataException;
import jsdai.SMachining_schema.AMachining_operation;
import jsdai.SMachining_schema.EAxis2_placement_3d;
import jsdai.SMachining_schema.EBoring_tool;
import jsdai.SMachining_schema.ECartesian_point;
import jsdai.SMachining_schema.ECutting_tool;
import jsdai.SMachining_schema.EDirection;
import jsdai.SMachining_schema.EDrilling;
import jsdai.SMachining_schema.EHand;
import jsdai.SMachining_schema.EMilling_machine_functions;
import jsdai.SMachining_schema.EMilling_technology;
import jsdai.SMachining_schema.EPlane;
import jsdai.SMachining_schema.ERound_hole;
import jsdai.SMachining_schema.EThrough_bottom_condition;
import jsdai.SMachining_schema.EToleranced_length_measure;
import jsdai.SMachining_schema.ETool_dimension;
import jsdai.lang.A_double;
import jsdai.lang.SdaiException;

public class STEPHoleProject extends STEPProject {

	protected ERound_hole round_hole;
	protected EDrilling drilling;
	protected EToleranced_length_measure toleranced_length_measure1;
	protected EMilling_technology milling_technology;
	protected EMilling_machine_functions milling_machine_functions;
	protected double HoleRay;
	protected String HoleId;
	protected ECutting_tool cutting_tool;
	protected EBoring_tool boring_tool;
	protected String ToolId;
	
	public STEPHoleProject() throws SdaiException {
		super();
	}


	public String getToolId() {
		return ToolId;
	}
	

	public String getHoleId() {
		return HoleId;
	}

	public void setHoleId(String holeId) {
		HoleId = holeId;
	}


	public double getHoleRay() {
		return HoleRay;
	}

	public void setHoleRay(double holeRay) {
		HoleRay = holeRay;
	}

	public void addHoleWorkingStep(String workingStepId, double feedrate,
			double x, double y, double z) throws SdaiException, MissingDataException {

		machining_workingstep = createWorkingStep(workingStepId);
		if (cutting_tool == null) {
			cutting_tool = (ECutting_tool) model
					.createEntityInstance(ECutting_tool.class);
			cutting_tool.setIts_id(null, getToolId());
		}
		ETool_dimension tool_dimension = (ETool_dimension) model
				.createEntityInstance(ETool_dimension.class);
		tool_dimension.setDiameter(null, (2*getHoleRay()));
		tool_dimension.setTool_top_angle(null, 0.00);
		if (boring_tool == null) {
			boring_tool = (EBoring_tool) model
					.createEntityInstance(EBoring_tool.class);
			boring_tool.setDimension(null, tool_dimension);
			boring_tool.setNumber_of_teeth(null, 3);
			boring_tool.setHand_of_cut(null, EHand.RIGHT);
			boring_tool.setCoolant_through_tool(null, false);
			boring_tool.setRetract_movement_forbidden(null, true);

		}

		cutting_tool.setIts_tool_body(null, boring_tool);
		cutting_tool.setOverall_assembly_length(null, 75.000);
		
		if (HoleId != null) {
			round_hole = (ERound_hole) model
					.createEntityInstance(ERound_hole.class);
			round_hole.setIts_id(null, HoleId);

			drilling = (EDrilling) model.createEntityInstance(EDrilling.class);
			drilling.setIts_id(null, HoleId);
		}else{
			throw new MissingDataException();
		}

		EAxis2_placement_3d axis2_placement_3d2 = (EAxis2_placement_3d) model
				.createEntityInstance(EAxis2_placement_3d.class);
		axis2_placement_3d2.setName(null, "Hole");

		ECartesian_point cartesian_point2 = (ECartesian_point) model
				.createEntityInstance(ECartesian_point.class);
		cartesian_point2.setName(null, "");

		EDirection direction2 = (EDirection) model
				.createEntityInstance(EDirection.class);
		direction2.setName(null, "");

		EDirection direction3 = (EDirection) model
				.createEntityInstance(EDirection.class);
		direction3.setName(null, "");

		A_double p2 = cartesian_point2.createCoordinates(null);
		p2.addByIndex(1, x);
		p2.addByIndex(2, y);
		p2.addByIndex(3, z);

		A_double d2 = direction2.createDirection_ratios(null);
		d2.addByIndex(1, 0.0);
		d2.addByIndex(2, 0.0);
		d2.addByIndex(3, 1.0);

		A_double d3 = direction3.createDirection_ratios(null);
		d3.addByIndex(1, 1.0);
		d3.addByIndex(2, 0.0);
		d3.addByIndex(3, 0.0);

		axis2_placement_3d2.setLocation(null, cartesian_point2);
		axis2_placement_3d2.setAxis(null, direction2);
		axis2_placement_3d2.setRef_direction(null, direction3);

		toleranced_length_measure1 = (EToleranced_length_measure) model
				.createEntityInstance(EToleranced_length_measure.class);
		toleranced_length_measure1.setTheoretical_size(null, 20.000);

		milling_technology = (EMilling_technology) model
				.createEntityInstance(EMilling_technology.class);

		milling_machine_functions = (EMilling_machine_functions) model
				.createEntityInstance(EMilling_machine_functions.class);

		milling_technology.setFeedrate(null, feedrate);
		milling_technology.setSynchronize_spindle_with_feed(null, false);
		milling_technology.setInhibit_feedrate_override(null, false);
		milling_technology.setInhibit_spindle_override(null, false);

		milling_machine_functions.setCoolant(null, true);
		milling_machine_functions.setMist(null, true);
		milling_machine_functions.setThrough_spindle_coolant(null, false);
		milling_machine_functions.setChip_removal(null, true);

		drilling.setRetract_plane(null, 10.000);
		drilling.setIts_technology(null, milling_technology);
		drilling.setIts_machine_functions(null, milling_machine_functions);
		drilling.setIts_tool(null, cutting_tool);

		EThrough_bottom_condition through_bottom_condition = (EThrough_bottom_condition) model
				.createEntityInstance(EThrough_bottom_condition.class);

		round_hole.setFeature_placement(null, axis2_placement_3d2);
		round_hole.setDiameter(null, toleranced_length_measure1);
		round_hole.setBottom_condition(null, through_bottom_condition);

		ECartesian_point cartesian_point1 = (ECartesian_point) model
				.createEntityInstance(ECartesian_point.class);
		cartesian_point1.setName(null, "");
		A_double p1 = cartesian_point1.createCoordinates(null);
		p1.addByIndex(1, 0.0);
		p1.addByIndex(2, 0.0);
		p1.addByIndex(3, 60.0);

		EAxis2_placement_3d axis2_placement_3d1 = (EAxis2_placement_3d) model
				.createEntityInstance(EAxis2_placement_3d.class);
		axis2_placement_3d1.setName(null, "");
		axis2_placement_3d1.setLocation(null, cartesian_point1);

		EPlane plane = (EPlane) model.createEntityInstance(EPlane.class);
		plane.setName(null, "");
		plane.setPosition(null, axis2_placement_3d1);

		AMachining_operation dril = round_hole.createIts_operations(null);
		dril.addUnordered(drilling, null);

		machining_workingstep.setIts_secplane(null, plane);
		machining_workingstep.setIts_feature(null, round_hole);
		machining_workingstep.setIts_operation(null, drilling);

		addWorkingStep(machining_workingstep);
	}

	public void relationateHoleVars() throws SdaiException {
		relationateVars();
		round_hole.setIts_workpiece(null, workpiece);
	}


	public void setToolId(String string) {
		ToolId = string;
		
	}

}
